@@ -133,7 +133,6 @@ process_property_notify (MetaCompositor	*compositor,
                          XPropertyEvent *event,
                          MetaWindow     *window)
 {
-  MetaDisplay *display = compositor->display;
   MutterWindow *cw;
 
   if (window == NULL)
@@ -150,13 +149,7 @@ process_property_notify (MetaCompositor	*compositor,
       DEBUG_TRACE ("process_property_notify: net_wm_window_opacity\n");
       return;
     }
-  else if (event->atom == meta_display_get_atom (display,
-					       META_ATOM__NET_WM_WINDOW_TYPE))
-    {
-      mutter_window_update_window_type (cw);
-      DEBUG_TRACE ("process_property_notify: net_wm_type\n");
-      return;
-    }
+
   DEBUG_TRACE ("process_property_notify: unknown\n");
 }
 
@@ -191,6 +184,12 @@ get_output_window (MetaScreen *screen)
   return output;
 }
 
+/**
+ * mutter_get_stage_for_screen:
+ * @screen: a #MetaScreen
+ *
+ * Returns: (transfer none): The #ClutterStage for the screen
+ */
 ClutterActor *
 mutter_get_stage_for_screen (MetaScreen *screen)
 {
@@ -202,6 +201,12 @@ mutter_get_stage_for_screen (MetaScreen *screen)
   return info->stage;
 }
 
+/**
+ * mutter_get_overlay_group_for_screen:
+ * @screen: a #MetaScreen
+ *
+ * Returns: (transfer none): The overlay group corresponding to @screen
+ */
 ClutterActor *
 mutter_get_overlay_group_for_screen (MetaScreen *screen)
 {
@@ -213,6 +218,12 @@ mutter_get_overlay_group_for_screen (MetaScreen *screen)
   return info->overlay_group;
 }
 
+/**
+ * mutter_get_window_group_for_screen:
+ * @screen: a #MetaScreen
+ *
+ * Returns: (transfer none): The window group corresponding to @screen
+ */
 ClutterActor *
 mutter_get_window_group_for_screen (MetaScreen *screen)
 {
@@ -224,6 +235,12 @@ mutter_get_window_group_for_screen (MetaScreen *screen)
   return info->window_group;
 }
 
+/**
+ * mutter_get_windows:
+ * @screen: a #MetaScreen
+ *
+ * Returns: (transfer none) (element-type Clutter.Actor): The windows on @screen
+ */
 GList *
 mutter_get_windows (MetaScreen *screen)
 {
@@ -592,7 +609,10 @@ is_grabbed_event (XEvent *event)
 
   return FALSE;
 }
-
+/**
+ * meta_compositor_process_event: (skip)
+ *
+ */
 gboolean
 meta_compositor_process_event (MetaCompositor *compositor,
                                XEvent         *event,
@@ -997,6 +1017,10 @@ mutter_repaint_func (gpointer data)
   return TRUE;
 }
 
+/**
+ * meta_compositor_new: (skip)
+ *
+ */
 MetaCompositor *
 meta_compositor_new (MetaDisplay *display)
 {
@@ -1034,6 +1058,10 @@ meta_compositor_new (MetaDisplay *display)
   return compositor;
 }
 
+/**
+ * mutter_get_overlay_window: (skip)
+ *
+ */
 Window
 mutter_get_overlay_window (MetaScreen *screen)
 {
