@@ -67,15 +67,14 @@ struct _MutterShapedTexturePrivate
   CoglHandle mask_texture;
   CoglHandle material;
   CoglHandle material_unshaped;
-#if 1 /* see workaround comment in mutter_shaped_texture_paint */
-  CoglHandle material_workaround;
-#endif
 
   MetaRegion *clip_region;
 
   guint mask_width, mask_height;
 
   GArray *rectangles;
+
+  guint create_mipmaps : 1;
 };
 
 static void
@@ -108,6 +107,7 @@ mutter_shaped_texture_init (MutterShapedTexture *self)
 
   priv->paint_tower = mutter_texture_tower_new ();
   priv->mask_texture = COGL_INVALID_HANDLE;
+  priv->create_mipmaps = TRUE;
 }
 
 static void
@@ -132,13 +132,6 @@ mutter_shaped_texture_dispose (GObject *object)
       cogl_handle_unref (priv->material_unshaped);
       priv->material_unshaped = COGL_INVALID_HANDLE;
     }
-#if 1 /* see comment in mutter_shaped_texture_paint */
-  if (priv->material_workaround != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->material_workaround);
-      priv->material_workaround = COGL_INVALID_HANDLE;
-    }
-#endif
 
   mutter_shaped_texture_set_clip_region (self, NULL);
 
@@ -173,8 +166,11 @@ mutter_shaped_texture_notify (GObject    *object,
       MutterShapedTexture *stex = (MutterShapedTexture *) object;
       MutterShapedTexturePrivate *priv = stex->priv;
 
-      mutter_texture_tower_set_base_texture (priv->paint_tower,
-					     clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex)));
+      mutter_shaped_texture_clear (stex);
+
+      if (priv->create_mipmaps)
+	mutter_texture_tower_set_base_texture (priv->paint_tower,
+					       clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex)));
     }
 }
 
@@ -196,6 +192,9 @@ mutter_shaped_texture_dirty_mask (MutterShapedTexture *stex)
 
       cogl_handle_unref (priv->mask_texture);
       priv->mask_texture = COGL_INVALID_HANDLE;
+
+      if (priv->material != COGL_INVALID_HANDLE)
+        cogl_material_set_layer (priv->material, 1, COGL_INVALID_HANDLE);
     }
 }
 
@@ -298,6 +297,10 @@ mutter_shaped_texture_paint (ClutterActor *actor)
   CoglHandle paint_tex;
   guint tex_width, tex_height;
   ClutterActorBox alloc;
+
+  static CoglHandle material_template = COGL_INVALID_HANDLE;
+  static CoglHandle material_unshaped_template = COGL_INVALID_HANDLE;
+
   CoglHandle material;
 
   if (priv->clip_region && meta_region_is_empty (priv->clip_region))
@@ -306,27 +309,25 @@ mutter_shaped_texture_paint (ClutterActor *actor)
   if (!CLUTTER_ACTOR_IS_REALIZED (CLUTTER_ACTOR (stex)))
     clutter_actor_realize (CLUTTER_ACTOR (stex));
 
-  /* If mipmaps are supported, then the texture filter quality will
-   * still be HIGH here. In that case we just want to use the base
-   * texture. If mipmaps are not support then
-   * on_glx_texture_pixmap_pre_paint() will have reset the texture
-   * filter quality to MEDIUM, and we should use the MutterTextureTower
-   * mipmap emulation.
+  /* The GL EXT_texture_from_pixmap extension does allow for it to be
+   * used together with SGIS_generate_mipmap, however this is very
+   * rarely supported. Also, even when it is supported there
+   * are distinct performance implications from:
    *
-   * http://bugzilla.openedhand.com/show_bug.cgi?id=1877 is an RFE
-   * for a better way of handling this.
+   *  - Updating mipmaps that we don't need
+   *  - Having to reallocate pixmaps on the server into larger buffers
    *
-   * While it would be nice to have direct access to the 'can_mipmap'
-   * boolean in ClutterGLXTexturePixmap, since since MutterTextureTower
-   * creates the scaled down images on demand there is no substantial
-   * overhead from doing the work to create and update the tower and
-   * not using it, other than the memory allocated for the MutterTextureTower
-   * structure itself.
+   * So, we just unconditionally use our mipmap emulation code. If we
+   * wanted to use SGIS_generate_mipmap, we'd have to  query COGL to
+   * see if it was supported (no API currently), and then if and only
+   * if that was the case, set the clutter texture quality to HIGH.
+   * Setting the texture quality to high without SGIS_generate_mipmap
+   * support for TFP textures will result in fallbacks to XGetImage.
    */
-  if (clutter_texture_get_filter_quality (CLUTTER_TEXTURE (stex)) == CLUTTER_TEXTURE_QUALITY_HIGH)
-    paint_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex));
-  else
+  if (priv->create_mipmaps)
     paint_tex = mutter_texture_tower_get_paint_texture (priv->paint_tower);
+  else
+    paint_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex));
 
   if (paint_tex == COGL_INVALID_HANDLE)
     return;
@@ -341,10 +342,14 @@ mutter_shaped_texture_paint (ClutterActor *actor)
     {
       /* If there are no rectangles use a single-layer texture */
 
-      if (priv->material_unshaped == COGL_INVALID_HANDLE)
-	priv->material_unshaped = cogl_material_new ();
+      if (priv->material_unshaped == COGL_INVALID_HANDLE) 
+        {
+          if (G_UNLIKELY (material_unshaped_template == COGL_INVALID_HANDLE))
+            material_unshaped_template = cogl_material_new ();
 
-      material = priv->material_unshaped;
+          priv->material_unshaped = cogl_material_copy (material_unshaped_template);
+        }
+        material = priv->material_unshaped;
     }
   else
     {
@@ -352,11 +357,14 @@ mutter_shaped_texture_paint (ClutterActor *actor)
 
       if (priv->material == COGL_INVALID_HANDLE)
 	{
-	  priv->material = cogl_material_new ();
-
-	  cogl_material_set_layer_combine (priv->material, 1,
+	   if (G_UNLIKELY (material_template == COGL_INVALID_HANDLE))
+	    {
+	      material_template =  cogl_material_new ();
+	      cogl_material_set_layer_combine (material_template, 1,
 					   "RGBA = MODULATE (PREVIOUS, TEXTURE[A])",
 					   NULL);
+	    }
+	  priv->material = cogl_material_copy (material_template);
 	}
       material = priv->material;
 
@@ -500,6 +508,65 @@ mutter_shaped_texture_new (void)
   return self;
 }
 
+void
+mutter_shaped_texture_set_create_mipmaps (MutterShapedTexture *stex,
+					  gboolean             create_mipmaps)
+{
+  MutterShapedTexturePrivate *priv;
+
+  g_return_if_fail (MUTTER_IS_SHAPED_TEXTURE (stex));
+
+  priv = stex->priv;
+
+  create_mipmaps = create_mipmaps != FALSE;
+
+  if (create_mipmaps != priv->create_mipmaps)
+    {
+      CoglHandle base_texture;
+
+      priv->create_mipmaps = create_mipmaps;
+
+      base_texture = create_mipmaps ?
+	clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (stex)) : COGL_INVALID_HANDLE;
+
+      mutter_texture_tower_set_base_texture (priv->paint_tower, base_texture);
+    }
+}
+
+/* This is a workaround for deficiencies in the hack tower:
+ *
+ * When we call clutter_x11_texture_pixmap_set_pixmap(tp, None),
+ * ClutterX11TexturePixmap knows that it has to get rid of the old texture, but
+ * clutter_texture_set_cogl_texture(texture, COGL_INVALID_HANDLE) isn't allowed, so
+ * it grabs the material for the texture and manually sets the texture in it. This means
+ * that the "cogl-texture" property isn't notified, so we don't find out about it.
+ *
+ * And if we keep the CoglX11TexturePixmap around after the X pixmap is freed, then
+ * we'll trigger X errors when we actually try to free it.
+ *
+ * The only correct thing to do here is to change our code to derive
+ * from ClutterActor and get rid of the inheritance hack tower.  Once
+ * we want to depend on Clutter-1.4 (which has CoglTexturePixmapX11),
+ * that will be very easy to do.
+ */
+void
+mutter_shaped_texture_clear (MutterShapedTexture *stex)
+{
+  MutterShapedTexturePrivate *priv;
+
+  g_return_if_fail (MUTTER_IS_SHAPED_TEXTURE (stex));
+
+  priv = stex->priv;
+
+  mutter_texture_tower_set_base_texture (priv->paint_tower, COGL_INVALID_HANDLE);
+
+  if (priv->material != COGL_INVALID_HANDLE)
+    cogl_material_set_layer (priv->material, 0, COGL_INVALID_HANDLE);
+
+  if (priv->material_unshaped != COGL_INVALID_HANDLE)
+    cogl_material_set_layer (priv->material_unshaped, 0, COGL_INVALID_HANDLE);
+}
+
 void
 mutter_shaped_texture_clear_rectangles (MutterShapedTexture *stex)
 {
