@@ -27,15 +27,11 @@
 #include "constraints.h"
 #include "workspace-private.h"
 #include "place.h"
+#include "prefs.h"
 
 #include <stdlib.h>
 #include <math.h>
 
-#define NOT_TOO_SMALL_BORDER 20    /* space around the resized window */
-#define NOT_TOO_SMALL_TRIGGER 0.60 /* percentage of screen size below which
-                                    * the NOT_TOO_SMALL constraint triggers
-                                    */
-
 #if 0
  // This is the short and sweet version of how to hack on this file; see
  // doc/how-constraints-works.txt for the gory details.  The basics of
@@ -102,12 +98,12 @@ typedef enum
   PRIORITY_ENTIRELY_VISIBLE_ON_WORKAREA = 1,
   PRIORITY_SIZE_HINTS_INCREMENTS = 1,
   PRIORITY_MAXIMIZATION = 2,
+  PRIORITY_TILING = 2,
   PRIORITY_FULLSCREEN = 2,
   PRIORITY_SIZE_HINTS_LIMITS = 3,
   PRIORITY_TITLEBAR_VISIBLE = 4,
   PRIORITY_PARTIALLY_VISIBLE_ON_WORKAREA = 4,
-  PRIORITY_NOT_TOO_SMALL = 5,
-  PRIORITY_MAXIMUM = 5 /* Dummy value used for loop end = max(all priorities) */
+  PRIORITY_MAXIMUM = 4 /* Dummy value used for loop end = max(all priorities) */
 } ConstraintPriority;
 
 typedef enum
@@ -145,11 +141,18 @@ typedef struct
   GList  *usable_screen_region;
   GList  *usable_monitor_region;
 } ConstraintInfo;
-
+static gboolean constrain_modal_dialog       (MetaWindow         *window,
+                                              ConstraintInfo     *info,
+                                              ConstraintPriority  priority,
+                                              gboolean            check_only);
 static gboolean constrain_maximization       (MetaWindow         *window,
                                               ConstraintInfo     *info,
                                               ConstraintPriority  priority,
                                               gboolean            check_only);
+static gboolean constrain_tiling             (MetaWindow         *window,
+                                              ConstraintInfo     *info,
+                                              ConstraintPriority  priority,
+                                              gboolean            check_only);
 static gboolean constrain_fullscreen         (MetaWindow         *window,
                                               ConstraintInfo     *info,
                                               ConstraintPriority  priority,
@@ -183,11 +186,6 @@ static gboolean constrain_partially_onscreen (MetaWindow         *window,
                                               ConstraintPriority  priority,
                                               gboolean            check_only);
 
-static gboolean constrain_not_too_small      (MetaWindow         *window,
-                                              ConstraintInfo     *info,
-                                              ConstraintPriority  priority,
-                                              gboolean            check_only);
-
 static void setup_constraint_info        (ConstraintInfo      *info,
                                           MetaWindow          *window,
                                           MetaFrameGeometry   *orig_fgeom,
@@ -220,7 +218,9 @@ typedef struct {
 } Constraint;
 
 static const Constraint all_constraints[] = {
+  {constrain_modal_dialog,       "constrain_modal_dialog"},
   {constrain_maximization,       "constrain_maximization"},
+  {constrain_tiling,             "constrain_tiling"},
   {constrain_fullscreen,         "constrain_fullscreen"},
   {constrain_size_increments,    "constrain_size_increments"},
   {constrain_size_limits,        "constrain_size_limits"},
@@ -229,8 +229,6 @@ static const Constraint all_constraints[] = {
   {constrain_fully_onscreen,     "constrain_fully_onscreen"},
   {constrain_titlebar_visible,   "constrain_titlebar_visible"},
   {constrain_partially_onscreen, "constrain_partially_onscreen"},
-  {constrain_not_too_small,      "constrain_not_too_small"},
-
   {NULL,                         NULL}
 };
 
@@ -434,11 +432,11 @@ setup_constraint_info (ConstraintInfo      *info,
     meta_workspace_get_onmonitor_region (cur_workspace, 
                                          monitor_info->number);
 
-#if 0
   /* Workaround braindead legacy apps that don't know how to
    * fullscreen themselves properly.
    */
-  if (meta_rectangle_equal (new, &monitor_info->rect) &&
+  if (meta_prefs_get_force_fullscreen() &&
+      meta_rectangle_equal (new, &monitor_info->rect) &&
       window->has_fullscreen_func &&
       !window->fullscreen)
     {
@@ -451,7 +449,6 @@ setup_constraint_info (ConstraintInfo      *info,
                   window->desc);
       meta_window_make_fullscreen_internal (window);
     }
-#endif
 
   /* Log all this information for debugging */
   meta_topic (META_DEBUG_GEOMETRY,
@@ -541,10 +538,11 @@ place_window_if_needed(MetaWindow     *window,
   if (window->placed || did_placement)
     {
       if (window->maximize_horizontally_after_placement ||
-          window->maximize_vertically_after_placement)
+          window->maximize_vertically_after_placement   ||
+          window->fullscreen_after_placement)
         {
-          /* define a sane saved_rect so that the user can unmaximize to
-           * something reasonable.
+          /* define a sane saved_rect so that the user can unmaximize or
+           * make unfullscreen to something reasonable.
            */
           if (info->current.width >= info->work_area_monitor.width)
             {
@@ -571,6 +569,15 @@ place_window_if_needed(MetaWindow     *window,
           if (window->frame && !window->fullscreen)
             meta_frame_calc_geometry (window->frame, info->fgeom);
 
+          if (window->fullscreen_after_placement)
+            {
+              window->saved_rect = info->current;
+              window->fullscreen = TRUE;
+              window->fullscreen_after_placement = FALSE;
+
+              g_object_notify (G_OBJECT (window), "fullscreen");
+            }
+
           window->maximize_horizontally_after_placement = FALSE;
           window->maximize_vertically_after_placement = FALSE;
         }
@@ -729,6 +736,48 @@ get_size_limits (const MetaWindow        *window,
     }
 }
 
+static gboolean
+constrain_modal_dialog (MetaWindow         *window,
+                        ConstraintInfo     *info,
+                        ConstraintPriority  priority,
+                        gboolean            check_only)
+{
+  int x, y;
+  MetaWindow *parent = meta_window_get_transient_for (window);
+  gboolean constraint_already_satisfied;
+
+  if (!meta_prefs_get_attach_modal_dialogs ())
+    return TRUE;
+  if (window->type != META_WINDOW_MODAL_DIALOG || !parent || parent == window)
+    return TRUE;
+
+  x = parent->rect.x + (parent->rect.width / 2  - info->current.width / 2);
+  y = 0;
+  if (parent->frame)
+    {
+      MetaFrameGeometry fgeom;
+
+      x += parent->frame->rect.x;
+      y += parent->frame->rect.y;
+
+      meta_frame_calc_geometry (parent->frame, &fgeom);
+      y += fgeom.top_height;
+
+      y += info->fgeom->top_height;
+    }
+  else
+    y = parent->rect.y + info->fgeom->top_height;
+
+  constraint_already_satisfied = (x == info->current.x) && (y == info->current.y);
+
+  if (check_only || constraint_already_satisfied)
+    return constraint_already_satisfied;
+
+  info->current.y = y;
+  info->current.x = x;
+  return TRUE;
+}
+
 static gboolean
 constrain_maximization (MetaWindow         *window,
                         ConstraintInfo     *info,
@@ -745,7 +794,8 @@ constrain_maximization (MetaWindow         *window,
     return TRUE;
 
   /* Determine whether constraint applies; exit if it doesn't */
-  if (!window->maximized_horizontally && !window->maximized_vertically)
+  if ((!window->maximized_horizontally && !window->maximized_vertically) ||
+      META_WINDOW_TILED (window))
     return TRUE;
 
   /* Calculate target_size = maximized size of (window + frame) */
@@ -813,6 +863,59 @@ constrain_maximization (MetaWindow         *window,
   return TRUE;
 }
 
+static gboolean
+constrain_tiling (MetaWindow         *window,
+                  ConstraintInfo     *info,
+                  ConstraintPriority  priority,
+                  gboolean            check_only)
+{
+  MetaRectangle target_size;
+  MetaRectangle min_size, max_size;
+  gboolean hminbad, vminbad;
+  gboolean horiz_equal, vert_equal;
+  gboolean constraint_already_satisfied;
+
+  if (priority > PRIORITY_TILING)
+    return TRUE;
+
+  /* Determine whether constraint applies; exit if it doesn't */
+  if (!META_WINDOW_TILED (window))
+    return TRUE;
+
+  /* Calculate target_size - as the tile previews need this as well, we
+   * use an external function for the actual calculation
+   */
+  meta_window_get_current_tile_area (window, &target_size);
+  unextend_by_frame (&target_size, info->fgeom);
+
+  /* Check min size constraints; max size constraints are ignored as for
+   * maximized windows.
+   */
+  get_size_limits (window, info->fgeom, FALSE, &min_size, &max_size);
+  hminbad = target_size.width < min_size.width;
+  vminbad = target_size.height < min_size.height;
+  if (hminbad || vminbad)
+    return TRUE;
+
+  /* Determine whether constraint is already satisfied; exit if it is */
+  horiz_equal = target_size.x      == info->current.x &&
+                target_size.width  == info->current.width;
+  vert_equal  = target_size.y      == info->current.y &&
+                target_size.height == info->current.height;
+  constraint_already_satisfied = horiz_equal && vert_equal;
+  if (check_only || constraint_already_satisfied)
+    return constraint_already_satisfied;
+
+  /*** Enforce constraint ***/
+  info->current.x      = target_size.x;
+  info->current.width  = target_size.width;
+  info->current.y      = target_size.y;
+  info->current.height = target_size.height;
+
+  return TRUE;
+}
+
+
 static gboolean
 constrain_fullscreen (MetaWindow         *window,
                       ConstraintInfo     *info,
@@ -864,7 +967,7 @@ constrain_size_increments (MetaWindow         *window,
 
   /* Determine whether constraint applies; exit if it doesn't */
   if (META_WINDOW_MAXIMIZED (window) || window->fullscreen || 
-      info->action_type == ACTION_MOVE)
+      META_WINDOW_TILED (window) || info->action_type == ACTION_MOVE)
     return TRUE;
 
   /* Determine whether constraint is already satisfied; exit if it is */
@@ -995,7 +1098,7 @@ constrain_aspect_ratio (MetaWindow         *window,
   constraints_are_inconsistent = minr > maxr;
   if (constraints_are_inconsistent ||
       META_WINDOW_MAXIMIZED (window) || window->fullscreen || 
-      info->action_type == ACTION_MOVE)
+      META_WINDOW_TILED (window) || info->action_type == ACTION_MOVE)
     return TRUE;
 
   /* Determine whether constraint is already satisfied; exit if it is.  We
@@ -1394,95 +1497,3 @@ constrain_partially_onscreen (MetaWindow         *window,
 
   return retval;
 }
-
-static gboolean
-constrain_not_too_small (MetaWindow         *window,
-                         ConstraintInfo     *info,
-                         ConstraintPriority  priority,
-                         gboolean            check_only)
-{
-  gboolean already_satisfied = TRUE;
-  gboolean resize = FALSE;
-  gint screen_width, screen_height, width, height;
-  gint x = 0, y = 0, old_x = 0, old_y;
-  MetaRectangle *start_rect;
-  MetaRectangle min_size, max_size;
-
-  if (priority > PRIORITY_NOT_TOO_SMALL || /* None of our businesss */
-      info->is_user_action ||              /* Don't mess users about */
-      window->user_placed    ||
-      window->type != META_WINDOW_NORMAL)  /* App windows only */
-    return TRUE;
-
-  old_x = info->current.x - info->fgeom->left_width;
-  old_y = info->current.y - info->fgeom->top_height;
-
-  width  = info->current.width  +
-    info->fgeom->left_width + info->fgeom->right_width;
-  height = info->current.height +
-    info->fgeom->top_height + info->fgeom->bottom_height;
-
-  screen_width  = info->work_area_monitor.width;
-  screen_height = info->work_area_monitor.height;
-
-  if (width == screen_width && height == screen_height)
-    return TRUE;
-
-  if ((((gfloat)width / (gfloat) screen_width) > NOT_TOO_SMALL_TRIGGER))
-    {
-      already_satisfied = FALSE;
-      resize = TRUE;
-    }
-
-  if (!resize)
-    {
-      /*
-       * If we are not resizing the application, attempt to center it.
-       */
-      x = (screen_width  - width) / 2;
-      y = (screen_height > height) ? (screen_height - height) / 2 : 0;
-
-      if (x != old_x || y != old_y)
-        already_satisfied = FALSE;
-    }
-
-  if (check_only || already_satisfied)
-    return already_satisfied;
-
-  if (resize)
-    {
-      width  = screen_width - 2 * NOT_TOO_SMALL_BORDER;
-      height = screen_height - 2 * NOT_TOO_SMALL_BORDER;
-
-      /* We respect both the max and min size hints */
-      /* We should include the frame here */
-      get_size_limits (window, info->fgeom, TRUE, &min_size, &max_size);
-
-      if (width > max_size.width)
-          width = max_size.width;
-      else if (width < min_size.width)
-          width = min_size.width;
-
-      if (height > max_size.height)
-          height = max_size.height;
-      else if (height < min_size.height)
-          height = min_size.height;
-
-      x = (screen_width  - width) / 2;
-      y = (screen_height > height) ? (screen_height - height) / 2 : 0;
-    }
-
-  if (info->action_type == ACTION_MOVE_AND_RESIZE)
-    start_rect = &info->current;
-  else
-    start_rect = &info->orig;
-
-  start_rect->x      = x;
-  start_rect->y      = y;
-  start_rect->width  = width;
-  start_rect->height = height;
-
-  unextend_by_frame (start_rect, info->fgeom);
-
-  return TRUE;
-}
