@@ -276,7 +276,7 @@ meta_frame_get_flags (MetaFrame *frame)
   if (META_WINDOW_ALLOWS_VERTICAL_RESIZE (frame->window))
     flags |= META_FRAME_ALLOWS_VERTICAL_RESIZE;
   
-  if (frame->window->has_focus)
+  if (meta_window_appears_focused (frame->window))
     flags |= META_FRAME_HAS_FOCUS;
 
   if (frame->window->shaded)
